import base64
import os
import pwd
import subprocess

from charmhelpers import fetch
from charmhelpers.core import hookenv
from charmhelpers.payload import (archive, execd)

SERVICE_NAME = 'snappy-device-agents'
AGENT_CONFIGFILE = 'spi-agent.conf'
DEVICE_CONFIGFILE = 'default.yaml'
REQUIRED_PACKAGES = [
    'python-virtualenv', 'python3-dev', 'ubuntu-device-flash',
    'openssh-client', 'sshpass'
]
DEFAULT_USER = 'ubuntu'

config = hookenv.config()


def _service_dir():
    template = '/srv/{}/{}'
    return template.format(config['environment'], SERVICE_NAME)


def _agent_dir():
    template = '{}/spi-agent'
    return template.format(_service_dir())


def basenode(service_name):
    hookenv.log('Executing basenode')
    execd.execd_preinstall()


def log_start(service_name):
    hookenv.log('%s starting', SERVICE_NAME)


def install_packages(service_name):
    hookenv.log('Installing dependencies...')
    fetch.add_source('ppa:snappy-dev/tools')
    fetch.configure_sources(update=True)
    fetch.apt_install(REQUIRED_PACKAGES, fatal=True)


def update_files(service_name):
    if config.changed('ssh-priv-key') or config.changed('ssh-pub-key'):
        hookenv.log('Updating ssh-keys')
        ssh_path = os.path.join('/home', DEFAULT_USER, '.ssh')
        try:
            # This is handled better in python3
            os.makedirs(ssh_path, mode=0700)
        except:
            pass
        file_content = base64.b64decode(config['ssh-priv-key'])
        file_path = os.path.join(ssh_path, 'id_rsa')
        with open(file_path, 'w') as f:
            f.write(file_content)
        os.chown(file_path, pwd.getpwnam(DEFAULT_USER).pw_uid, -1)
        os.chmod(file_path, 0600)
        file_content = base64.b64decode(config['ssh-pub-key'])
        file_path = os.path.join(ssh_path, 'id_rsa.pub')
        with open(file_path, 'w') as f:
            f.write(file_content)
        os.chown(file_path, pwd.getpwnam(DEFAULT_USER).pw_uid, -1)

    if config.changed('provisioning-kit-tarball'):
        hookenv.log(
            'Updating provisioning-kit')
        file_content = base64.b64decode(config['provisioning-kit-tarball'])
        file_path = os.path.join('/tmp', 'snappy-device-agents.tgz')
        with open(file_path, 'w') as f:
            f.write(file_content)
        archive.extract_tarfile(file_path, os.path.dirname(_service_dir()))

    if config.changed('spi-agent-tarball'):
        # The spi-agent gets extracted under the provisioning kit dir
        hookenv.log(
            'Updating spi-agent')
        file_content = base64.b64decode(config['spi-agent-tarball'])
        file_path = os.path.join('/tmp', 'spi-agent.tgz')
        with open(file_path, 'w') as f:
            f.write(file_content)
        archive.extract_tarfile(file_path, os.path.dirname(_agent_dir()))

    if config.changed('pip-cache-tarball'):
        hookenv.log(
            'Updating pip-cache')
        file_content = base64.b64decode(config['pip-cache-tarball'])
        file_path = os.path.join('/tmp', 'pip-cache.tgz')
        with open(file_path, 'w') as f:
            f.write(file_content)
        archive.extract_tarfile(file_path, _service_dir())

    if config.changed('agent-configfile'):
        hookenv.log(
            'Updating service configuration file: %s', AGENT_CONFIGFILE)
        file_content = base64.b64decode(config['agent-configfile'])
        file_path = os.path.join(_service_dir(), AGENT_CONFIGFILE)
        with open(file_path, 'w') as f:
            f.write(file_content)

    if config.changed('agent-configfile'):
        hookenv.log(
            'Updating service configuration file: %s', DEVICE_CONFIGFILE)
        file_content = base64.b64decode(config['device-configfile'])
        file_path = os.path.join(_service_dir(), DEVICE_CONFIGFILE)
        with open(file_path, 'w') as f:
            f.write(file_content)


def install_python_packages(service_name):
    env_dir = os.path.join(_service_dir(), 've')
    if os.path.exists(env_dir):
        hookenv.log('Service venv already exists, nothing to do ...')
        return

    hookenv.log('Installing python packages into the venv ...')
    subprocess.check_call(['virtualenv', '-p', 'python3', env_dir])

    pip_cache = os.path.join(_service_dir(), 'pip-cache')
    requirements = os.path.join(_service_dir(), 'requirements.txt')

    hookenv.log('Installing from download cache.')
    subprocess.check_call(['%s/bin/pip' % env_dir,
                           'install',
                           '--no-index',
                           '--find-links={}'.format(pip_cache),
                           '-r', requirements])

    subprocess.check_call(['%s/bin/pip' % env_dir,
                           'install', '--no-deps', '-e', _service_dir()])
